#include "widgets/ClockWidget.h"

ClockWidget::ClockWidget(QWidget *parent) :
    QWidget(parent)
{

	_paused = true;
	_seconds = 0;
	/* resolution in dots per centimeter */
	qreal dpiY = qreal(this->physicalDpiY())/2.54;
	
	/* set divisionHeight to  a half centimeter on the display device */
	int divisionHeight = dpiY*0.5;
	
	_renderFont.setPixelSize(divisionHeight);
	
	int minWidth = QFontMetrics(_renderFont).width("000:00") + 10;
	
	this->setMinimumHeight(divisionHeight+10);
	this->setMinimumWidth(minWidth);

	this->startTimer(1000);
}


void ClockWidget::paintEvent(QPaintEvent *event)
{
	QRect rect = this->rect();
	
	QTextOption tOption;
	tOption.setAlignment(Qt::AlignCenter);
	
	int minutes = _seconds / 60;
	int seconds = _seconds - minutes*60;
	
	
	QString minutesString = QString::number(minutes);
	if(minutesString.length() < 2)
		minutesString.prepend("0");
	if(minutesString.length() < 3)
		minutesString.prepend("0");
	
	QString secondsString = QString::number(seconds);
	if(secondsString.size() < 2)
		secondsString.prepend("0");
	
	QPainter painter;
	painter.begin(this);
	painter.setRenderHint(QPainter::Antialiasing, true);
	painter.setRenderHint(QPainter::TextAntialiasing, true);


	QLinearGradient grad;
	grad.setColorAt(0.0, QColor(0,0,0,140));
	grad.setColorAt(1.0, QColor(0,0,0,220));
	grad.setStart(QPointF(0.0, 0.0));
	grad.setFinalStop(QPoint(0.0, this->height()));
	QBrush flowBrush(grad);
	QPainterPath path;

	path.addRoundedRect(3, /* center horizontal */
											3, /* center vertical */
											this->width()-2*3,
											this->height()-2*3, 
											5, 5); 
	painter.fillPath(path, flowBrush);


	
	painter.setFont(_renderFont);
	painter.setPen(QPen(Qt::white));
	painter.drawText(rect, minutesString+":"+secondsString, tOption);
	painter.end();
	event->accept();
}

void ClockWidget::timerEvent(QTimerEvent *event)
{
	
	event->accept();
	
	if(_paused)
		return;
	
	_seconds++;
	
	this->update();
}

void ClockWidget::mouseDoubleClickEvent(QMouseEvent *event)
{
	event->accept();
	_seconds = 0;
	this->update();
}

void ClockWidget::mouseReleaseEvent(QMouseEvent *event)
{
	event->accept();
	_paused = !_paused;
		
}

QSize ClockWidget::sizeHint() const
{
	return this->minimumSize();
}
