#include "NotesViewportWidget.h"
#include "nPresenter.h"

NotesViewportWidget::NotesViewportWidget(QWidget *parent) :
    ViewportWidget(parent)
{
}

Document::Viewports NotesViewportWidget::viewport()
{
	return Document::NotesViewport;
}

QSize NotesViewportWidget::sizeHint() const
{
	return QWidget::sizeHint();
}

QSize NotesViewportWidget::viewportSize()
{
	return ViewportWidget::viewportSize();
}

void NotesViewportWidget::paintEvent(QPaintEvent *event)
{

	ViewportWidget::paintEvent(event);

	QPainter painter;
	painter.begin(this);
	painter.setRenderHint(QPainter::Antialiasing, true);
	painter.setRenderHint(QPainter::TextAntialiasing, true);
	
	
	QPoint center = this->rect().center();
	
	QRect rect;
	rect.setSize(QSize(50, 50));

	QFont font;
	font.setPixelSize(rect.height()-10);

	painter.setFont(font);
	if(_blackoutEnabled)
	{

		QPoint centerF = center;
		centerF.setX(center.x()+29);
		rect.moveCenter(centerF);
		
		QPen pen(Qt::red);
		pen.setWidthF(3.0);
		
		painter.setPen(pen);
		painter.setBrush(Qt::black);
		painter.setOpacity(0.7);
		painter.drawRoundedRect(rect, 5, 5);

		painter.setOpacity(1.0);
		painter.setPen(QPen(Qt::white));
		QTextOption option;
		option.setAlignment(Qt::AlignCenter);
		painter.drawText(rect, "B", option);

	}

	if(_freezeEnabled)
	{
		
		QPoint centerF = center;
		centerF.setX(center.x()-29);
		rect.moveCenter(centerF);
		QPen pen(Qt::red);
		pen.setWidthF(3.0);
		painter.setPen(pen);
		painter.setBrush(QBrush(QColor(30, 120, 255)));
		painter.setOpacity(0.6);
		painter.drawRoundedRect(rect, 5, 5);

		painter.setOpacity(1.0);
		painter.setPen(QPen(Qt::white));
		QTextOption option;
		option.setAlignment(Qt::AlignCenter);
		painter.drawText(rect, "F", option);

	}
	
	/* draw help screen as long as no document is loaded */
	if(!nPresenter::document().loaded())
	{
		QString help;
		help += QString("nPresenter - Help: \n\n")
						+ QString("\t\tpress Key:\n")
						+ QString("\t\t - O\t-\tto open a PDF File\n\n")
						+ QString("\t\t - C\t-\tto show/hide clock\n")
						+ QString("\t\t - B\t-\tto blackout presentation screen\n")
						+ QString("\t\t - F\t-\tto freeze the current page on presentation screen\n")
						+ QString("\t\t - L\t-\tto change Layout of PDF\n")
						+ QString("\n\n\tClock:\n")
						+ QString("\t\t - Click - Start/Stop\n")
						+ QString("\t\t - Doubleclick - Reset\n");
		
		QRect helpScreen = this->rect();
		helpScreen.setSize(this->size() -QSize(10,10));
		helpScreen.moveTop(5);
		helpScreen.moveLeft(5);
				
		
		painter.setOpacity(0.7);
		painter.setBrush(Qt::black);
		painter.setPen(QPen());
		painter.drawRoundedRect(helpScreen, 10.0, 10.0);
		
		painter.setPen(QPen(Qt::white));
		painter.setOpacity(1.0);

		QRect helpText = this->rect();
		helpText.setSize(this->size() -QSize(30,30));
		helpText.moveTop(15);
		helpText.moveLeft(15);

				QTextOption tOption;
		tOption.setAlignment(Qt::AlignVCenter | Qt::AlignLeft);
		tOption.setWrapMode(QTextOption::NoWrap);
		
		QFont font;
		painter.setFont(font);
		painter.drawText(helpText, help, tOption);
		
	}
	
	painter.end();
}
