#include "widgets/PresentationViewWidget.h"
#include "nPresenter.h"

PresentationViewportWidget::PresentationViewportWidget(QWidget *parent) :
    ViewportWidget(parent)
{

	QDesktopWidget * desktop = qApp->desktop();
	
	/* we only handle auto fullscreen if the desktop has two screens */
	if(desktop->numScreens() <= 1 && desktop->numScreens() < 3)
		return;
	
	int primScreen = desktop->primaryScreen();
	int secScreen = primScreen+1;
	
	if(secScreen >= 2)
		secScreen = 0;
	
	this->setGeometry(desktop->availableGeometry(secScreen));
	
	this->setWindowFlags(Qt::Window | Qt::CustomizeWindowHint | Qt::WindowMaximizeButtonHint);
}

const QImage PresentationViewportWidget::slide()
{
	return QImage();
}

Document::Viewports PresentationViewportWidget::viewport()
{
	return Document::PresentationOneViewport;
}

//QSize PresentationViewportWidget::sizeHint() const
//{
//	return QWidget::sizeHint();
//}

void PresentationViewportWidget::paintEvent(QPaintEvent *event)
{
		QPainter painter;
		painter.begin(this);
		painter.setRenderHint(QPainter::Antialiasing);
		painter.setClipRect(event->rect());
		painter.setClipping(true);

		QRect wRect;
		wRect.setSize(this->size());

		QPoint center = wRect.center();

		QSize vpSize = this->viewportSize();

		QRect targetRect;
		targetRect.setSize(vpSize);
		targetRect.moveCenter(center);

		painter.fillRect(this->rect(), Qt::black);

		if(!_blackoutEnabled)
			this->drawSlide(painter, targetRect, _viewportCache, 1.0, 0.0);

		painter.end();
}

void PresentationViewportWidget::viewportIsReady(Viewports viewport, int page, QSize size)
{
	if(!_freezeEnabled)
		ViewportWidget::viewportIsReady(viewport, page, size);
}

void PresentationViewportWidget::enableFreeze(bool enable)
{
	/* if freeze was enabled, request the current page */
	if(!enable)
		this->requestPage(nPresenter::currentPage());	
	
	ViewportWidget::enableFreeze(enable);
}
