#include "widgets/PreviewCurrentPageWidget.h"
#include "nPresenter.h"

PreviewCurrentPageWidget::PreviewCurrentPageWidget(QWidget *parent) :
    ViewportWidget(parent)
{

	QDesktopWidget * desktop = qApp->desktop();
	
	/* we only handle auto fullscreen if the desktop has two screens */
	if(desktop->numScreens() <= 1 && desktop->numScreens() < 3)
		return;
	
	/* there are exactly one primary and one secondary screen.
		 so we place the presentation viewport into the second screen */
	
	int primScreen = desktop->primaryScreen();
	int secScreen = primScreen+1;
	
	/* the secondary screen is the physical first one */
	if(secScreen >= 2)
		secScreen = 0;
	
	/* tell presentation view that it should get as big as possible */
	this->setGeometry(desktop->availableGeometry(secScreen));
	
	/* this window can be maximized, minimized but not closed. */
	this->setWindowFlags(Qt::Window | Qt::CustomizeWindowHint | Qt::WindowMaximizeButtonHint);
}

const QImage PreviewCurrentPageWidget::slide()
{
	return QImage();
}

Document::Viewports PreviewCurrentPageWidget::viewport()
{
	return Document::PresentationOneViewport;
}

//QSize PreviewCurrentPageWidget::sizeHint() const
//{
//	return QWidget::sizeHint();
//}

void PreviewCurrentPageWidget::paintEvent(QPaintEvent *event)
{
	event->accept();

	QPainter painter;
	painter.begin(this);
	
	painter.setRenderHint(QPainter::Antialiasing, true);
	painter.setRenderHint(QPainter::SmoothPixmapTransform, true);
	
	painter.setClipRect(event->rect());
	painter.setClipping(true);

	/* center of the widget */
	QPoint center = this->rect().center();
	
	/* set the target rect to viewportSize() and center it */
	QRect targetRect;
	targetRect.setSize(this->viewportSize());
	
	/* Set target rect to size of _viewportCache
		 if _viewportCache is smaller than the viewport. */
	if(_viewportCache.size().width() <= targetRect.width() &&
		 _viewportCache.size().height() <= targetRect.height() && !_viewportCache.isNull())
		targetRect.setSize(_viewportCache.size());
	
	/* center the image to draw */
	targetRect.moveCenter(center);
	targetRect.moveTop(5);
	
	/* draw page */
	this->drawSlide(painter, targetRect, _viewportCache, 1.0, 10.0);
	
	/* draw a orange border around the image to indicate that it is the active one */
	QPen pen(QColor("#ff5900"));
	pen.setWidthF(3.0);
	painter.setBrush(QBrush());
	painter.setPen(pen);
	painter.drawRoundedRect(targetRect, 10.0, 10.0);
	
	
	painter.end();
	
}


