#ifndef SECTIONVIEWER_H
#define SECTIONVIEWER_H

#include <QtCore/QtCore>
#include <QtGui/QtGui>
#include <QtXml/QtXml>

#include "widgets/SectionViewer/SectionWidget.h"

class SectionViewer : public QWidget
{
	Q_OBJECT

	QDomDocument * _toc;
	QList<SectionWidget*> _sections;
	Qt::Orientation _orientation;
	int _currentPage;

	public:
		explicit SectionViewer(QWidget *parent = 0);

		QSize sizeHint() const;

		Qt::Orientation orientation();
	protected:
		virtual void paintEvent(QPaintEvent * event);
		void resizeEvent(QResizeEvent * event);
		

	signals:
		
		/** @brief emitted when user selects a page */
		void pageChanged(int pageNum);
	
	public slots:
		
		/** @brief call when the active page changes */
		void setActivePage(int pageNum);

		/** @brief index the table of contents of the current document */
		void updateSections();

		/** @brief calculate layout of sections */
		void layoutSections();

		void changeDockArea(Qt::DockWidgetArea area);
		void changeTopLevel(bool topLevel);

};


class SectionViewerDock : public QDockWidget
{
		Q_OBJECT;
	public:
		explicit SectionViewerDock(QWidget * parent = 0)
			: QDockWidget(parent)
		{
			
		}
		
	protected:
		void paintEvent(QPaintEvent *event)
		{
			QWidget::paintEvent(event);
		}
};

#endif // SECTIONVIEWER_H
