#include "SectionWidget.h"
#include "nPresenter.h"

SectionWidget::SectionWidget(QDomElement & section, QWidget * parent) :
    QWidget(parent)
{
	_section = section;
	_isActive = false;
	_currentPage = 0;
	
	this->calculatePageAndPageSpan();
	
	
	//	qDebug() << this << section.tagName() << this->page();
	/* setup subsections */

	QDomElement tmpSubsection = _section.firstChildElement();
	while(!tmpSubsection.isNull())
	{

		SubsectionWidget * widget = new SubsectionWidget(tmpSubsection, this);
		if(!widget)
			continue;
		widget->setParent(this);
		
		_subsections.append(widget);
		tmpSubsection = tmpSubsection.nextSiblingElement();
	}
	
	
}

void SectionWidget::calculatePageAndPageSpan()
{
	
	QString pageDest = QString("Destination/")+_section.attribute("Destination");
	
	if(_section.attribute("Destination").isNull())
		pageDest = QString("DestinationName/")+_section.attribute("DestinationName");
	
	/* -1 because pdf pages are starting with 1, not with 0 */
	_page = nPresenter::document().pageNumberByBookmark(pageDest)-1;
	
	
	if(_page < 0) _page = 0;
	
	QDomElement nextSection = _section.nextSiblingElement();
	
	int nextPage = nPresenter::document().numPages();
	
	
	/* we are not the last section */
	if(!nextSection.isNull())
	{
		QString nextPageDest = QString("Destination/")+nextSection.attribute("Destination");
		
		if(nextSection.attribute("Destination").isNull())
			nextPageDest = QString("DestinationName/")+nextSection.attribute("DestinationName");
		
		/* -1 because pdf pages are starting with 1, not with 0 */
		nextPage = nPresenter::document().pageNumberByBookmark(nextPageDest)-1;
	}
	
	_pageSpan = nextPage-_page;
	
	if(_pageSpan <= 0)
		_pageSpan = 1;
	
}

int SectionWidget::page()
{
	return _page;
}

int SectionWidget::pageSpan()
{
	return _pageSpan;
}

bool SectionWidget::containsPage(int pageNum)
{
	return (pageNum >= this->page() && pageNum < this->page()+this->pageSpan());
}

void SectionWidget::paintEvent(QPaintEvent *event)
{
	
	
	event->accept();

	QPainter painter;
	
	painter.begin(this);
	painter.setRenderHint(QPainter::Antialiasing);
	painter.setRenderHint(QPainter::TextAntialiasing);
	
	QPen textPen;
	textPen.setWidth(0);
	
	QPen pen(Qt::white);
	pen.setWidthF(3.0);
	
	QBrush brush(QColor(0,0,0,0));
	
	if(_isActive)
	{
		QColor khkColor("#ff5900");
		khkColor.setAlphaF(1.0);
		brush.setColor(Qt::white);
		textPen.setColor(Qt::black);
		pen.setColor(QColor("#ff5900"));
	}
	else
	{
		brush.setColor(QColor(255,255,255,200));
		textPen.setColor(Qt::black);
		pen.setColor(QColor(0,0,0,0));
	}
	
	painter.setPen(pen);
	painter.setBrush(brush);

	QRect border;
	border.setSize(_divisionSize);
	painter.drawRoundedRect(border, 5, 5);


	QRect activeBorder(this->rect());
	painter.setBrush(QBrush());
	painter.drawRoundedRect(activeBorder, 5, 5);
	
	QTextOption tOption;
	tOption.setAlignment(Qt::AlignCenter);
	tOption.setWrapMode(QTextOption::NoWrap);
	
	QFont font;
	font.setPixelSize(_divisionSize.height()-2);
	font.setBold(true);
	
	painter.setPen(textPen);
	painter.setFont(font);

	border.setSize(border.size()-QSize(4,0));
	border.moveTopLeft(QPoint(2,2));
	
	QString sectionName = _section.tagName();
	sectionName = QFontMetrics(font).elidedText(sectionName, Qt::ElideRight, border.width());
	
	painter.drawText(border, sectionName, tOption);
	
	
	if(!_section.hasChildNodes())
	{
		
		QString text = QString::number(this->pageSpan());
		if(_isActive)
			text.prepend(QString::number(_currentPage-this->page()+1)+"/");
		
		tOption.setAlignment(Qt::AlignCenter);
		painter.setPen(QPen(Qt::white));
		if(_isActive)
			painter.setPen(QColor("#ff5900"));
		painter.drawText(this->rect(), text, tOption);
		
	}
	
	painter.end();
	
}

void SectionWidget::resizeEvent(QResizeEvent *event)
{
	
	
	/* bypass warning of not used varialbe*/
	if(event){};
	
	/* get the number of displayable subsections */
	int numVisibleSubsections = (this->height()-_divisionSize.height())/_divisionSize.height();
	
	/* setup viewport dimension */
	QRect viewPort;
	viewPort.setWidth(_divisionSize.width());
	viewPort.setHeight(_divisionSize.height()*numVisibleSubsections);
	viewPort.moveTop(_divisionSize.height()+5);
	
	/* we don't have any childs so we have nothing to do */
	if(_subsections.size() < 1)
		return;
	
	
	/* by default we show the first subsections */
	int activeSubsection = 0;
	
	/* if this section is begind current page, scroll to the last subsection */
	if(this->page() < _currentPage)
		activeSubsection = _subsections.size()-1;
	
	/* or scroll to the active subsection, if this section contains the active subsection */
	for(int i = 0; i <_subsections.size(); i++)	
	{
		if(_subsections[i]->containsPage(_currentPage))
		{
			activeSubsection = i;
			break;
		}
	}
	
	
	/* calculate the scrollOffset.
		 +2 is due to index starts at zero and that we want to start to scroll whenever
		 the active subsection is the second last subsection in the viewport */
	int scrollOffset = (activeSubsection+2)-numVisibleSubsections;
	
	/* we don't need to scroll here */
	if(scrollOffset < 0 || _subsections.size()<=numVisibleSubsections)
		scrollOffset = 0;
	
	/* dont scroll the last item and only prevent this if we need to scroll*/
	if(activeSubsection+1 >= _subsections.size() && _subsections.size()>numVisibleSubsections)
		scrollOffset -= 1;
	
	for(int i = 0; i < _subsections.size(); i++)
	{
		
		_subsections[i]->changeDivisionSize(_divisionSize);
		_subsections[i]->resize(_divisionSize);
		_subsections[i]->move(0, 5+_divisionSize.height()*(1+i-scrollOffset));
		_subsections[i]->setVisible(true);
		
		/* if widget is not inside viewport hide it */
		if(!viewPort.contains(_subsections[i]->geometry()))
			_subsections[i]->setVisible(false);
		
	}
}

void SectionWidget::setActivePage(int pageNum)
{
	_isActive = this->containsPage(pageNum);
	_currentPage = pageNum;
	
	/* propagate current active page to subsections */
	for(int i = 0; i < _subsections.size(); i++)
		_subsections[i]->setActivePage(pageNum);
	this->resizeEvent(0);
	this->update();	
}

void SectionWidget::changeDivisionSize(QSize divisionSize)
{
	_divisionSize = divisionSize;
}
