#include "SubsectionWidget.h"
#include "nPresenter.h"

#include "widgets/SectionViewer/SectionWidget.h"

SubsectionWidget::SubsectionWidget(QDomElement & subsection, QWidget * parent) :
    QWidget(parent)
{
	_subsection = subsection;
	_isActive = false;
	_currentPage = 0;
	
	this->calculatePageAndPageSpan();
	//qDebug() << this << subsection.tagName() << this->page() << this->pageSpan();
}

void SubsectionWidget::calculatePageAndPageSpan()
{
	
	
	QString pageDest = QString("Destination/")+_subsection.attribute("Destination");
	
	if(_subsection.attribute("Destination").isNull())
		pageDest = QString("DestinationName/")+_subsection.attribute("DestinationName");
	
	/* -1 because pdf pages are starting with 1, not with 0 */
	_page = nPresenter::document().pageNumberByBookmark(pageDest)-1;
	
	
	if(_page < 0) _page = 0;
	
	QDomElement nextSection = _subsection.nextSiblingElement();
	
	
	int nextPage = nPresenter::document().numPages();
	
	/* we are the last subsection. go up in the tree */
	if(nextSection.isNull())
		nextSection = _subsection.parentNode().nextSiblingElement();
	
	
	if(!nextSection.isNull())
	{
		QString nextPageDest = QString("Destination/")+nextSection.attribute("Destination");
		
		if(nextSection.attribute("Destination").isNull())
			nextPageDest = QString("DestinationName/")+nextSection.attribute("DestinationName");
		
		/* -1 because pdf pages are starting with 1, not with 0 */
		nextPage = nPresenter::document().pageNumberByBookmark(nextPageDest)-1;
	}
	
	_pageSpan = nextPage-_page;
	
	if(_pageSpan <= 0)
		_pageSpan = 1;
}

int SubsectionWidget::page()
{
	return _page;
}

int SubsectionWidget::pageSpan()
{
	return _pageSpan;
}

bool SubsectionWidget::containsPage(int pageNum)
{
	return (pageNum >= this->page() && pageNum < this->page()+this->pageSpan());
}

void SubsectionWidget::paintEvent(QPaintEvent *event)
{
	
	event->accept();

	QPainter painter;
	
	painter.begin(this);
	painter.setRenderHint(QPainter::Antialiasing);
	painter.setRenderHint(QPainter::TextAntialiasing);
	
	QPoint center = this->rect().center();
	
	QPen pen;
	pen.setWidth(0);
	
	QBrush brush(QColor(0,0,0,255));

	if(_isActive)
	{
		pen.setColor("#ff5900");
		brush.setColor("#ff5900");
	}
	else
	{
		pen.setColor(Qt::white);
		brush.setColor(Qt::white);
	}
	
	QRect border;
	border.setSize(_divisionSize);
	
	border.moveCenter(center);

	QPen rectPen;
	rectPen.setWidthF(0.0);
	rectPen.setColor(QColor(0,0,0,0));

	painter.setPen(rectPen);

	painter.drawRoundedRect(border, 5, 5);

	QRect enumeratorRect; //(0, center.y()-border.height()/2, border.height()/2, border.height()/2);
	enumeratorRect.setSize(QSize(border.height()/2, border.height()/2));
	enumeratorRect.moveLeft(enumeratorRect.height()/2);
	enumeratorRect.moveTop(enumeratorRect.height()/2);

	painter.setBrush(brush);
	painter.drawRoundedRect(enumeratorRect, 2,2);
	

	QFont font;
	font.setPixelSize(_divisionSize.height()-2);
	font.setBold(true);
	
	border.moveRight(border.height()+5);
	border.setWidth(border.width()-border.height());
	
	QFont statsFont;
	statsFont.setPixelSize(_divisionSize.height()-4);
	statsFont.setBold(true);
	QString statsString;
	
	/* if subsections contains more than one pages inform the user about the page span */
	if(this->pageSpan() > 1)
		statsString = QString::number(this->pageSpan());
	if(!statsString.isNull() && _isActive )
		statsString.prepend(QString::number(_currentPage-this->page()+1)+"/");
		
	int statsWidth = QFontMetrics(statsFont).width(statsString);
	
	
	QRect statsRect;
	statsRect.setWidth(statsWidth);
	statsRect.setHeight(statsFont.pixelSize());
	statsRect.moveRight(border.width());
	
	QRect textRect;
	textRect.setX(border.height());
	textRect.setY(border.y());
	textRect.setSize(border.size() - QSize(border.height() +statsWidth, 0));

	QString sectionName = _subsection.tagName();
	sectionName = QFontMetrics(font).elidedText(sectionName, Qt::ElideRight, textRect.width());

	QTextOption tOption;
	
	painter.setPen(pen);
	painter.setFont(font);
	tOption.setAlignment(Qt::AlignVCenter);
	tOption.setWrapMode(QTextOption::NoWrap);
	painter.drawText(textRect, sectionName, tOption);
	
	tOption.setAlignment(Qt::AlignRight);
	tOption.setWrapMode(QTextOption::NoWrap);
	painter.setFont(statsFont);
	painter.drawText(statsRect, statsString, tOption);
	
	painter.end();
	
}

void SubsectionWidget::resizeEvent(QResizeEvent *event)
{
	/* bypass warning */
	event->type();
}

void SubsectionWidget::mouseReleaseEvent(QMouseEvent *event)
{
	event->accept();
	emit this->pageChanged(this->page());
}

void SubsectionWidget::setActivePage(int pageNum)
{
	_isActive = this->containsPage(pageNum);
	_currentPage = pageNum;
	this->update();	
}

void SubsectionWidget::changeDivisionSize(QSize divisionSize)
{
	_divisionSize = divisionSize;
}
