#ifndef SUBSECTIONWIDGET_H
#define SUBSECTIONWIDGET_H

#include <QtCore/QtCore>
#include <QtGui/QtGui>
#include <QtXml/QtXml>



class SubsectionWidget : public QWidget
{
	Q_OBJECT
	
	private:
		/** the size of one division inside section */
		QSize _divisionSize;
		
		QDomElement _subsection;
		
		int _page;
		int _pageSpan;
		
		bool _isActive;
		int _currentPage;
		
	private:
		void calculatePageAndPageSpan();
		
	public:
		explicit SubsectionWidget(QDomElement & subsection, QWidget * parent);
		
		/** @returns the page number where this section starts */
		int page();
		
		/** @brief calculates page span */
		int pageSpan();
		
		/** @brief checks whether section contains pageNumber */
		bool containsPage(int pageNum);

		
	protected:
		void resizeEvent(QResizeEvent * event);
		void paintEvent(QPaintEvent * event);
		void mouseReleaseEvent(QMouseEvent * event);
		
	signals:
	
		/** signal is emmited when this or a child widget changed the current page */
		void pageChanged(int pageNum);

	public slots:
		/** @brief the current active pagenumber	*/
		void setActivePage(int pageNum);
		void changeDivisionSize(QSize divisionSize);
	
};

#endif // SUBSECTIONWIDGET_H
