#include "ViewportWidget.h"
#include "nPresenter.h"

ViewportWidget::ViewportWidget(QWidget *parent) :
    QWidget(parent)
{
	this->setMinimumSize(200, 150);
}

void ViewportWidget::resizeEvent(QResizeEvent *event)
{
	event->accept();
	emit viewportSizeChanged(this);
}

void ViewportWidget::paintEvent(QPaintEvent *event)
{

	event->accept();

	QPainter painter;
	painter.begin(this);
	painter.setRenderHint(QPainter::Antialiasing);
	painter.setClipRect(event->rect());
	painter.setClipping(true);

	QPoint center = this->rect().center();
	
	QSize vpSize = this->viewportSize();
	
	QRect targetRect;
	targetRect.setSize(vpSize);
	targetRect.moveCenter(center);
	targetRect.moveTop(5);
	
	this->drawSlide(painter, targetRect, _viewportCache, 1.0, 10.0);
	painter.end();
}

void ViewportWidget::drawSlide(QPainter & painter, 
															 QRect where,
															 QImage & slide,
															 qreal opacity,
															 qreal roundedCorners,
															 QPen pen)
{
	int centerX = where.width()/2 + where.x();
	int centerY = where.height()/2 + where.y();
	
	/* - check whether image is too big */
	QSize imgSize = slide.size();
	
	/* aspect ratio */
	qreal imgAR = qreal(imgSize.width())/qreal(imgSize.height());
	
	/* check whether image's width is too big for destination. if so, scale down */
	if(imgSize.width() > where.width())
	{
		imgSize.setWidth(where.width());
		imgSize.setHeight(qreal(where.width())*(1.0/imgAR));
	}
	
	
	/* check whether image's height is too big for destination. if so, scale down */
	if(imgSize.height() > where.height())
	{
		imgSize.setHeight(where.height());
		imgSize.setWidth(qRound(qreal(where.height())*imgAR));
	}		
	
	/* image with rounded corners */
	QPainterPath path;
	int dx = centerX-imgSize.width()/2;
	int dy = centerY-imgSize.height()/2;
	path.addRoundedRect(dx, /* center horizontal */
											dy, /* center vertical */
											imgSize.width(),
											imgSize.height(), 
											roundedCorners, roundedCorners); 
	
	/* setup brush */
	QBrush imageBrush;
	imageBrush.setTextureImage(slide);
	
	/* transform Image-Brush to the center of the viewport */
	QTransform imageTransform;
	imageTransform.translate(dx, dy);
	qreal imageScale = qreal(imgSize.width())/qreal(slide.width());
	imageTransform.scale(imageScale, imageScale);
	imageBrush.setTransform(imageTransform);
	
	/* apply all configs to painter and draw image */
	painter.save();
	painter.setPen(pen);
	painter.setBrush(imageBrush);
	painter.setOpacity(opacity);
	painter.drawPath(path);
	painter.restore();
	
}

void ViewportWidget::timerEvent(QTimerEvent *event)
{
	event->accept();
}

QSize ViewportWidget::viewportSize()
{
	QSize viewportSize = this->size();
	QRect widgetBox = this->rect();
	
	/* aspect ratio */
	//qreal vpAspectRation = this->viewport().width()/this->viewport().height();
	
	/** @todo extract from page together with viewport */
	qreal vpAspectRation = 4.0/3.0;
	
	viewportSize.setHeight(qreal(viewportSize.width())*(1.0/vpAspectRation));
	
	/* check whether image's height is too big for viewport. if so, scale down */
	if(viewportSize.height() > widgetBox.height())
	{
		viewportSize.setHeight(widgetBox.height());
		viewportSize.setWidth(qRound(qreal(widgetBox.height())*vpAspectRation));
	}	
	
	return viewportSize;
}



void ViewportWidget::requestPage(int pageNum)
{
	nPresenter::document().requestViewport(this->viewport(),
														pageNum,
														this->viewportSize());
}

void ViewportWidget::viewportIsReady(Document::Viewports viewport, int page, QSize size)
{
	if(viewport == this->viewport() && page == nPresenter::currentPage() && size == this->viewportSize())
	{
		_viewportCache = nPresenter::document().viewport(viewport, page, size);
		this->update();
	}
}

void ViewportWidget::enableBlackout(bool enable)
{
	_blackoutEnabled = enable;
	this->update();
}

void ViewportWidget::enableFreeze(bool enable)
{
	_freezeEnabled = enable;
	this->update();
}
