#ifndef VIEWPORTWIDGET_H
#define VIEWPORTWIDGET_H

#include <QtCore/QtCore>
#include <QtGui/QtGui>

#include "pdf/Document.h"

/** @brief displays excerpt from the presentation */
class ViewportWidget : public QWidget
{
	Q_OBJECT
	private:	


	protected:
		
		QImage _viewportCache;
		
		bool _blackoutEnabled;
		bool _freezeEnabled;
		
		

	public:
	    explicit ViewportWidget(QWidget *parent = 0);
			typedef Document::Viewports Viewports;
	
	protected:	
		
		
		void drawSlide(QPainter & painter,
									 QRect where,
									 QImage & slide,
									 qreal opacity = 1.0,
									 qreal roundedCorners = 0.0,
									 QPen pen = QPen(QColor(0,0,0,0)));
		
		virtual void resizeEvent(QResizeEvent * event);
		virtual void paintEvent(QPaintEvent * event);
		virtual void timerEvent(QTimerEvent * event);
		
		/** @return viewport of this widget */
		virtual Document::Viewports viewport() = 0;
		
		/** @brief returns the viewpot size 
		* this is not the same as size(): Instead the maximum area 
		* according to the aspect ratio of viewport() is returned
		*/
		virtual QSize viewportSize();		
	signals:
		void viewportSizeChanged(ViewportWidget * which);
		
	public slots:
		
		
		/** 
		 * this slot among others is connected to nPresenter::currentPageChanged().
		 * Use this function to request relevant pages from Document to be rendered
		 * @see Document::requestViewport()
		 */
		virtual void requestPage(int pageNum);
		
		/** 
		 * This slot is connected to Document::requestedViewportReady().
		 * Use this functiont to check whether the finished request is relevant for this widget
		 * and fetch the rendered viewport with Document::viewport() if apropriate.
		 * @see Document::viewport()
		 */
		virtual void viewportIsReady(Viewports viewport, int page, QSize size);

		
		void enableBlackout(bool enable);
		void enableFreeze(bool enable);
		
		
		

};

#endif // VIEWPORTWIDGET_H
