#include "WaitingIndicator.h"

WaitingIndicator::WaitingIndicator(QWidget *parent) :
    QWidget(parent)
{
	_steps = 16;
	
	_opacity = 0.0;
	_deltaOpacity = 0.0;
	
	_phase = 0.0;
	
	_timerId = -1;
	
	/* standard size */
	this->resize(64, 64);
}

void WaitingIndicator::paintEvent(QPaintEvent *event)
{
	/* draw indicator */
	
	int centerX = this->width()/2;
	int centerY = this->height()/2;
	
	QPainter painter;
	
	painter.begin(this);
	painter.setRenderHint(QPainter::Antialiasing);
	painter.save();
	
	QPen pen;
	pen.setColor(QColor("#ff5900"));
	pen.setWidthF(3.0);
	pen.setCapStyle(Qt::RoundCap);
	painter.setPen(pen);
	
	QPainterPath path;
	path.addRoundedRect(centerX-this->width()/2+2,
											centerY-this->height()/2+2,
											this->width() -4,
											this->height()-4,
											10, 10);
	
	painter.setBrush(QBrush(Qt::black));
	painter.setOpacity(0.4*_opacity);
	painter.drawPath(path);
	
	/* draw white rotating lines */
	pen.setColor(Qt::white);
	painter.setPen(pen);
	
	
	for(int i = 0; i < _steps; i ++)
	{
		qreal currPhase = _phase + qreal(i)/qreal(_steps);
		qreal innerX = 5.0*cos(2.0*M_PI*currPhase)+centerX;
		qreal innerY = 5.0*sin(2.0*M_PI*currPhase)+centerY;
		
		qreal outerX = 15.0*cos(2.0*M_PI*currPhase)+centerX;
		qreal outerY = 15.0*sin(2.0*M_PI*currPhase)+centerY;
			
		qreal opacity = 1.0/qreal(_steps)*qreal(i);
		painter.setOpacity(opacity*_opacity);
			
		painter.drawLine(innerX, innerY, outerX, outerY);
	
	}

	painter.restore();
	painter.end();
	
}

void WaitingIndicator::timerEvent(QTimerEvent *event)
{
	
	_opacity += _deltaOpacity;
	
	if(_opacity > 1.0)
		_opacity = 1.0;
	
	_phase += 1.0/qreal(_steps)*2.0;
	_phase -= floor(_phase);
	
	if(_opacity < 0.0)
	{
		this->killTimer(_timerId);
		_timerId = -1;
	}
	this->update();
	
}

void WaitingIndicator::show()
{
	_phase = 0.0;
	_opacity = 0.0;
	_deltaOpacity = 0.33;
	
	if(_timerId < 0)
		_timerId = this->startTimer(100);
}

void WaitingIndicator::hide()
{
	_deltaOpacity = -0.33;
}
